/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2018 Erik Moqvist
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/**
 * This file was generated by cantools version 29.6.0 Sat Dec  1 09:19:46 2018.
 */

#include <string.h>

#include "multiplex_2.h"

#define UNUSED(x) (void)(x)

ssize_t multiplex_2_shared_encode(
    uint8_t *dst_p,
    const struct multiplex_2_shared_t *src_p,
    size_t size)
{
    uint8_t s0;
    uint8_t s1;
    uint8_t s2;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(&dst_p[0], 0, 8);

    s0 = (uint8_t)src_p->s0;
    dst_p[0] |= ((s0 << 0) & 0x0fu);

    switch (src_p->s0) {

    case 1:
        s1 = (uint8_t)src_p->s1;
        dst_p[0] |= ((s1 << 4) & 0xf0u);
        break;

    case 2:
        s2 = (uint8_t)src_p->s2;
        dst_p[1] |= ((s2 << 0) & 0xffu);
        break;

    case 3:
        s1 = (uint8_t)src_p->s1;
        dst_p[0] |= ((s1 << 4) & 0xf0u);
        s2 = (uint8_t)src_p->s2;
        dst_p[1] |= ((s2 << 0) & 0xffu);
        break;

    case 4:
        s2 = (uint8_t)src_p->s2;
        dst_p[1] |= ((s2 << 0) & 0xffu);
        break;

    case 5:
        s2 = (uint8_t)src_p->s2;
        dst_p[1] |= ((s2 << 0) & 0xffu);
        break;

    default:
        break;
    }

    return (8);
}

int multiplex_2_shared_decode(
    struct multiplex_2_shared_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    uint8_t s0;
    uint8_t s1;
    uint8_t s2;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(dst_p, 0, sizeof(*dst_p));

    s0 = 0u;
    s0 |= ((uint8_t)(src_p[0] & 0x0fu) >> 0);

    if ((s0 & (1u << 3)) != 0u) {
        s0 |= 0xf0u;
    }

    dst_p->s0 = (int8_t)s0;

    switch (dst_p->s0) {

    case 1:
        s1 = 0u;
        s1 |= ((uint8_t)(src_p[0] & 0xf0u) >> 4);

        if ((s1 & (1u << 3)) != 0u) {
            s1 |= 0xf0u;
        }

        dst_p->s1 = (int8_t)s1;
        break;

    case 2:
        s2 = 0u;
        s2 |= ((uint8_t)(src_p[1] & 0xffu) >> 0);
        dst_p->s2 = (int8_t)s2;
        break;

    case 3:
        s1 = 0u;
        s1 |= ((uint8_t)(src_p[0] & 0xf0u) >> 4);

        if ((s1 & (1u << 3)) != 0u) {
            s1 |= 0xf0u;
        }

        dst_p->s1 = (int8_t)s1;
        s2 = 0u;
        s2 |= ((uint8_t)(src_p[1] & 0xffu) >> 0);
        dst_p->s2 = (int8_t)s2;
        break;

    case 4:
        s2 = 0u;
        s2 |= ((uint8_t)(src_p[1] & 0xffu) >> 0);
        dst_p->s2 = (int8_t)s2;
        break;

    case 5:
        s2 = 0u;
        s2 |= ((uint8_t)(src_p[1] & 0xffu) >> 0);
        dst_p->s2 = (int8_t)s2;
        break;

    default:
        break;
    }

    return (0);
}

bool multiplex_2_shared_s0_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_shared_s1_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_shared_s2_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

ssize_t multiplex_2_normal_encode(
    uint8_t *dst_p,
    const struct multiplex_2_normal_t *src_p,
    size_t size)
{
    uint8_t s0;
    uint8_t s1;
    uint8_t s2;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(&dst_p[0], 0, 8);

    s0 = (uint8_t)src_p->s0;
    dst_p[0] |= ((s0 << 0) & 0x0fu);

    switch (src_p->s0) {

    case 0:
        s1 = (uint8_t)src_p->s1;
        dst_p[0] |= ((s1 << 4) & 0xf0u);
        break;

    case 1:
        s2 = (uint8_t)src_p->s2;
        dst_p[1] |= ((s2 << 0) & 0xffu);
        break;

    default:
        break;
    }

    return (8);
}

int multiplex_2_normal_decode(
    struct multiplex_2_normal_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    uint8_t s0;
    uint8_t s1;
    uint8_t s2;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(dst_p, 0, sizeof(*dst_p));

    s0 = 0u;
    s0 |= ((uint8_t)(src_p[0] & 0x0fu) >> 0);

    if ((s0 & (1u << 3)) != 0u) {
        s0 |= 0xf0u;
    }

    dst_p->s0 = (int8_t)s0;

    switch (dst_p->s0) {

    case 0:
        s1 = 0u;
        s1 |= ((uint8_t)(src_p[0] & 0xf0u) >> 4);

        if ((s1 & (1u << 3)) != 0u) {
            s1 |= 0xf0u;
        }

        dst_p->s1 = (int8_t)s1;
        break;

    case 1:
        s2 = 0u;
        s2 |= ((uint8_t)(src_p[1] & 0xffu) >> 0);
        dst_p->s2 = (int8_t)s2;
        break;

    default:
        break;
    }

    return (0);
}

bool multiplex_2_normal_s0_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_normal_s1_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_normal_s2_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

ssize_t multiplex_2_extended_encode(
    uint8_t *dst_p,
    const struct multiplex_2_extended_t *src_p,
    size_t size)
{
    uint16_t s3;
    uint32_t s4;
    uint32_t s5;
    uint32_t s7;
    uint8_t s0;
    uint8_t s1;
    uint8_t s2;
    uint8_t s6;
    uint8_t s8;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(&dst_p[0], 0, 8);

    s0 = (uint8_t)src_p->s0;
    dst_p[0] |= ((s0 << 0) & 0x0fu);
    s6 = (uint8_t)src_p->s6;
    dst_p[4] |= ((s6 << 0) & 0xffu);

    switch (src_p->s0) {

    case 0:
        s1 = (uint8_t)src_p->s1;
        dst_p[0] |= ((s1 << 4) & 0xf0u);

        switch (src_p->s1) {

        case 0:
            s2 = (uint8_t)src_p->s2;
            dst_p[1] |= ((s2 << 0) & 0xffu);
            s3 = (uint16_t)src_p->s3;
            dst_p[2] |= ((s3 << 0) & 0xffu);
            dst_p[3] |= ((s3 >> 8) & 0xffu);
            break;

        case 2:
            s4 = (uint32_t)src_p->s4;
            dst_p[1] |= ((s4 << 0) & 0xffu);
            dst_p[2] |= ((s4 >> 8) & 0xffu);
            dst_p[3] |= ((s4 >> 16) & 0xffu);
            break;

        default:
            break;
        }
        break;

    case 1:
        s5 = (uint32_t)src_p->s5;
        dst_p[0] |= ((s5 << 4) & 0xf0u);
        dst_p[1] |= ((s5 >> 4) & 0xffu);
        dst_p[2] |= ((s5 >> 12) & 0xffu);
        dst_p[3] |= ((s5 >> 20) & 0xffu);
        break;

    default:
        break;
    }

    switch (src_p->s6) {

    case 1:
        s7 = (uint32_t)src_p->s7;
        dst_p[5] |= ((s7 << 0) & 0xffu);
        dst_p[6] |= ((s7 >> 8) & 0xffu);
        dst_p[7] |= ((s7 >> 16) & 0xffu);
        break;

    case 2:
        s8 = (uint8_t)src_p->s8;
        dst_p[5] |= ((s8 << 0) & 0xffu);
        break;

    default:
        break;
    }

    return (8);
}

int multiplex_2_extended_decode(
    struct multiplex_2_extended_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    uint16_t s3;
    uint32_t s4;
    uint32_t s5;
    uint32_t s7;
    uint8_t s0;
    uint8_t s1;
    uint8_t s2;
    uint8_t s6;
    uint8_t s8;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(dst_p, 0, sizeof(*dst_p));

    s0 = 0u;
    s0 |= ((uint8_t)(src_p[0] & 0x0fu) >> 0);

    if ((s0 & (1u << 3)) != 0u) {
        s0 |= 0xf0u;
    }

    dst_p->s0 = (int8_t)s0;
    s6 = 0u;
    s6 |= ((uint8_t)(src_p[4] & 0xffu) >> 0);
    dst_p->s6 = (int8_t)s6;

    switch (dst_p->s0) {

    case 0:
        s1 = 0u;
        s1 |= ((uint8_t)(src_p[0] & 0xf0u) >> 4);

        if ((s1 & (1u << 3)) != 0u) {
            s1 |= 0xf0u;
        }

        dst_p->s1 = (int8_t)s1;

        switch (dst_p->s1) {

        case 0:
            s2 = 0u;
            s2 |= ((uint8_t)(src_p[1] & 0xffu) >> 0);
            dst_p->s2 = (int8_t)s2;
            s3 = 0u;
            s3 |= ((uint16_t)(src_p[2] & 0xffu) >> 0);
            s3 |= ((uint16_t)(src_p[3] & 0xffu) << 8);
            dst_p->s3 = (int16_t)s3;
            break;

        case 2:
            s4 = 0u;
            s4 |= ((uint32_t)(src_p[1] & 0xffu) >> 0);
            s4 |= ((uint32_t)(src_p[2] & 0xffu) << 8);
            s4 |= ((uint32_t)(src_p[3] & 0xffu) << 16);

            if ((s4 & (1u << 23)) != 0u) {
                s4 |= 0xff000000u;
            }

            dst_p->s4 = (int32_t)s4;
            break;

        default:
            break;
        }
        break;

    case 1:
        s5 = 0u;
        s5 |= ((uint32_t)(src_p[0] & 0xf0u) >> 4);
        s5 |= ((uint32_t)(src_p[1] & 0xffu) << 4);
        s5 |= ((uint32_t)(src_p[2] & 0xffu) << 12);
        s5 |= ((uint32_t)(src_p[3] & 0xffu) << 20);

        if ((s5 & (1u << 27)) != 0u) {
            s5 |= 0xf0000000u;
        }

        dst_p->s5 = (int32_t)s5;
        break;

    default:
        break;
    }

    switch (dst_p->s6) {

    case 1:
        s7 = 0u;
        s7 |= ((uint32_t)(src_p[5] & 0xffu) >> 0);
        s7 |= ((uint32_t)(src_p[6] & 0xffu) << 8);
        s7 |= ((uint32_t)(src_p[7] & 0xffu) << 16);

        if ((s7 & (1u << 23)) != 0u) {
            s7 |= 0xff000000u;
        }

        dst_p->s7 = (int32_t)s7;
        break;

    case 2:
        s8 = 0u;
        s8 |= ((uint8_t)(src_p[5] & 0xffu) >> 0);
        dst_p->s8 = (int8_t)s8;
        break;

    default:
        break;
    }

    return (0);
}

bool multiplex_2_extended_s0_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s5_is_in_range(int32_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s1_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s4_is_in_range(int32_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s2_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s3_is_in_range(int16_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s6_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s8_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_s7_is_in_range(int32_t value)
{
    UNUSED(value);

    return (true);
}

ssize_t multiplex_2_extended_types_encode(
    uint8_t *dst_p,
    const struct multiplex_2_extended_types_t *src_p,
    size_t size)
{
    uint32_t s10;
    uint32_t s9;
    uint8_t s0;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(&dst_p[0], 0, 8);

    dst_p[0] |= ((src_p->s11 << 0) & 0x1fu);

    switch (src_p->s11) {

    case 3:
        s0 = (uint8_t)src_p->s0;
        dst_p[1] |= ((s0 << 0) & 0x0fu);

        switch (src_p->s0) {

        case 0:
            memcpy(&s10, &src_p->s10, sizeof(s10));
            dst_p[2] |= ((s10 << 0) & 0xffu);
            dst_p[3] |= ((s10 >> 8) & 0xffu);
            dst_p[4] |= ((s10 >> 16) & 0xffu);
            dst_p[5] |= ((s10 >> 24) & 0xffu);
            break;

        default:
            break;
        }
        break;

    case 5:
        memcpy(&s9, &src_p->s9, sizeof(s9));
        dst_p[3] |= ((s9 << 0) & 0xffu);
        dst_p[4] |= ((s9 >> 8) & 0xffu);
        dst_p[5] |= ((s9 >> 16) & 0xffu);
        dst_p[6] |= ((s9 >> 24) & 0xffu);
        break;

    default:
        break;
    }

    return (8);
}

int multiplex_2_extended_types_decode(
    struct multiplex_2_extended_types_t *dst_p,
    const uint8_t *src_p,
    size_t size)
{
    uint32_t s10;
    uint32_t s9;
    uint8_t s0;

    if (size < 8) {
        return (-EINVAL);
    }

    memset(dst_p, 0, sizeof(*dst_p));

    dst_p->s11 |= ((uint8_t)(src_p[0] & 0x1fu) >> 0);

    switch (dst_p->s11) {

    case 3:
        s0 = 0u;
        s0 |= ((uint8_t)(src_p[1] & 0x0fu) >> 0);

        if ((s0 & (1u << 3)) != 0u) {
            s0 |= 0xf0u;
        }

        dst_p->s0 = (int8_t)s0;

        switch (dst_p->s0) {

        case 0:
            s10 = 0u;
            s10 |= ((uint32_t)(src_p[2] & 0xffu) >> 0);
            s10 |= ((uint32_t)(src_p[3] & 0xffu) << 8);
            s10 |= ((uint32_t)(src_p[4] & 0xffu) << 16);
            s10 |= ((uint32_t)(src_p[5] & 0xffu) << 24);
            memcpy(&dst_p->s10, &s10, sizeof(dst_p->s10));
            break;

        default:
            break;
        }
        break;

    case 5:
        s9 = 0u;
        s9 |= ((uint32_t)(src_p[3] & 0xffu) >> 0);
        s9 |= ((uint32_t)(src_p[4] & 0xffu) << 8);
        s9 |= ((uint32_t)(src_p[5] & 0xffu) << 16);
        s9 |= ((uint32_t)(src_p[6] & 0xffu) << 24);
        memcpy(&dst_p->s9, &s9, sizeof(dst_p->s9));
        break;

    default:
        break;
    }

    return (0);
}

bool multiplex_2_extended_types_s11_is_in_range(uint8_t value)
{
    return ((value >= 2u) && (value <= 6u));
}

bool multiplex_2_extended_types_s0_is_in_range(int8_t value)
{
    UNUSED(value);

    return (true);
}

bool multiplex_2_extended_types_s10_is_in_range(float value)
{
    return ((value >= -340000000000000000000000000000000000000.0f) && (value <= 340000000000000000000000000000000000000.0f));
}

bool multiplex_2_extended_types_s9_is_in_range(float value)
{
    return ((value >= -1.34f) && (value <= 1235.0f));
}
