/**
 * The MIT License (MIT)
 *
 * Copyright (c) 2018 Erik Moqvist
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use, copy,
 * modify, merge, publish, distribute, sublicense, and/or sell copies
 * of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS
 * BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN
 * ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

/**
 * This file was generated by cantools version 29.6.0 Sat Dec  1 09:19:46 2018.
 */

#ifndef SIGNED_H
#define SIGNED_H

#include <stdint.h>
#include <stdbool.h>
#include <unistd.h>

#ifndef EINVAL
#    define EINVAL -22
#endif

#define SIGNED_MESSAGE32BIG_FRAME_ID (0x05u)
#define SIGNED_MESSAGE33BIG_FRAME_ID (0x04u)
#define SIGNED_MESSAGE64BIG_FRAME_ID (0x03u)
#define SIGNED_MESSAGE64_FRAME_ID (0x02u)
#define SIGNED_MESSAGE33_FRAME_ID (0x01u)
#define SIGNED_MESSAGE32_FRAME_ID (0x00u)



/**
 * Signals in message Message32big.
 *
 * @param s32big Value as on the CAN bus.
 *            Range: -
 *            Scale: 1
 *            Offset: 0
 */
struct signed_message32big_t {
    int32_t s32big;
};

/**
 * Signals in message Message33big.
 *
 * @param s33big Value as on the CAN bus.
 *            Range: -
 *            Scale: 1
 *            Offset: 0
 */
struct signed_message33big_t {
    int64_t s33big;
};

/**
 * Signals in message Message64big.
 *
 * @param s64big Value as on the CAN bus.
 *            Range: -
 *            Scale: 1
 *            Offset: 0
 */
struct signed_message64big_t {
    int64_t s64big;
};

/**
 * Signals in message Message64.
 *
 * @param s64 Value as on the CAN bus.
 *            Range: -9223372036854780000..9223372036854780000 (-9.22337203685478E+18..9.22337203685478E+18 -)
 *            Scale: 1
 *            Offset: 0
 */
struct signed_message64_t {
    int64_t s64;
};

/**
 * Signals in message Message33.
 *
 * @param s33 Value as on the CAN bus.
 *            Range: -4294967296..4294967295 (-4294967296..4294967295 -)
 *            Scale: 1
 *            Offset: 0
 */
struct signed_message33_t {
    int64_t s33;
};

/**
 * Signals in message Message32.
 *
 * @param s32 Value as on the CAN bus.
 *            Range: -
 *            Scale: 1
 *            Offset: 0
 */
struct signed_message32_t {
    int32_t s32;
};

/**
 * Encode message Message32big.
 *
 * @param[out] dst_p Buffer to encode the message into.
 * @param[in] src_p Data to encode.
 * @param[in] size Size of dst_p.
 *
 * @return Size of encoded data, or negative error code.
 */
ssize_t signed_message32big_encode(
    uint8_t *dst_p,
    const struct signed_message32big_t *src_p,
    size_t size);

/**
 * Decode message Message32big.
 *
 * @param[out] dst_p Object to decode the message into.
 * @param[in] src_p Message to decode.
 * @param[in] size Size of src_p.
 *
 * @return zero(0) or negative error code.
 */
int signed_message32big_decode(
    struct signed_message32big_t *dst_p,
    const uint8_t *src_p,
    size_t size);

/**
 * Check that given signal is in allowed range.
 *
 * @param[in] value Signal to check.
 *
 * @return true if in range, false otherwise.
 */
bool signed_message32big_s32big_is_in_range(int32_t value);

/**
 * Encode message Message33big.
 *
 * @param[out] dst_p Buffer to encode the message into.
 * @param[in] src_p Data to encode.
 * @param[in] size Size of dst_p.
 *
 * @return Size of encoded data, or negative error code.
 */
ssize_t signed_message33big_encode(
    uint8_t *dst_p,
    const struct signed_message33big_t *src_p,
    size_t size);

/**
 * Decode message Message33big.
 *
 * @param[out] dst_p Object to decode the message into.
 * @param[in] src_p Message to decode.
 * @param[in] size Size of src_p.
 *
 * @return zero(0) or negative error code.
 */
int signed_message33big_decode(
    struct signed_message33big_t *dst_p,
    const uint8_t *src_p,
    size_t size);

/**
 * Check that given signal is in allowed range.
 *
 * @param[in] value Signal to check.
 *
 * @return true if in range, false otherwise.
 */
bool signed_message33big_s33big_is_in_range(int64_t value);

/**
 * Encode message Message64big.
 *
 * @param[out] dst_p Buffer to encode the message into.
 * @param[in] src_p Data to encode.
 * @param[in] size Size of dst_p.
 *
 * @return Size of encoded data, or negative error code.
 */
ssize_t signed_message64big_encode(
    uint8_t *dst_p,
    const struct signed_message64big_t *src_p,
    size_t size);

/**
 * Decode message Message64big.
 *
 * @param[out] dst_p Object to decode the message into.
 * @param[in] src_p Message to decode.
 * @param[in] size Size of src_p.
 *
 * @return zero(0) or negative error code.
 */
int signed_message64big_decode(
    struct signed_message64big_t *dst_p,
    const uint8_t *src_p,
    size_t size);

/**
 * Check that given signal is in allowed range.
 *
 * @param[in] value Signal to check.
 *
 * @return true if in range, false otherwise.
 */
bool signed_message64big_s64big_is_in_range(int64_t value);

/**
 * Encode message Message64.
 *
 * @param[out] dst_p Buffer to encode the message into.
 * @param[in] src_p Data to encode.
 * @param[in] size Size of dst_p.
 *
 * @return Size of encoded data, or negative error code.
 */
ssize_t signed_message64_encode(
    uint8_t *dst_p,
    const struct signed_message64_t *src_p,
    size_t size);

/**
 * Decode message Message64.
 *
 * @param[out] dst_p Object to decode the message into.
 * @param[in] src_p Message to decode.
 * @param[in] size Size of src_p.
 *
 * @return zero(0) or negative error code.
 */
int signed_message64_decode(
    struct signed_message64_t *dst_p,
    const uint8_t *src_p,
    size_t size);

/**
 * Check that given signal is in allowed range.
 *
 * @param[in] value Signal to check.
 *
 * @return true if in range, false otherwise.
 */
bool signed_message64_s64_is_in_range(int64_t value);

/**
 * Encode message Message33.
 *
 * @param[out] dst_p Buffer to encode the message into.
 * @param[in] src_p Data to encode.
 * @param[in] size Size of dst_p.
 *
 * @return Size of encoded data, or negative error code.
 */
ssize_t signed_message33_encode(
    uint8_t *dst_p,
    const struct signed_message33_t *src_p,
    size_t size);

/**
 * Decode message Message33.
 *
 * @param[out] dst_p Object to decode the message into.
 * @param[in] src_p Message to decode.
 * @param[in] size Size of src_p.
 *
 * @return zero(0) or negative error code.
 */
int signed_message33_decode(
    struct signed_message33_t *dst_p,
    const uint8_t *src_p,
    size_t size);

/**
 * Check that given signal is in allowed range.
 *
 * @param[in] value Signal to check.
 *
 * @return true if in range, false otherwise.
 */
bool signed_message33_s33_is_in_range(int64_t value);

/**
 * Encode message Message32.
 *
 * @param[out] dst_p Buffer to encode the message into.
 * @param[in] src_p Data to encode.
 * @param[in] size Size of dst_p.
 *
 * @return Size of encoded data, or negative error code.
 */
ssize_t signed_message32_encode(
    uint8_t *dst_p,
    const struct signed_message32_t *src_p,
    size_t size);

/**
 * Decode message Message32.
 *
 * @param[out] dst_p Object to decode the message into.
 * @param[in] src_p Message to decode.
 * @param[in] size Size of src_p.
 *
 * @return zero(0) or negative error code.
 */
int signed_message32_decode(
    struct signed_message32_t *dst_p,
    const uint8_t *src_p,
    size_t size);

/**
 * Check that given signal is in allowed range.
 *
 * @param[in] value Signal to check.
 *
 * @return true if in range, false otherwise.
 */
bool signed_message32_s32_is_in_range(int32_t value);

#endif
